/**
   @file spiasm.h
  
   @author Tom Montemarano
  
   @version 0.5
  
   @copyright 
   copyright (c) Tom Montemarano 2014, All Rights MIT Licensed
  
   @brief Fast spi driver
@n  Uses PASM from xSPIAsm.spin, modified from SPI_Asm.spin propeller tool library object.
  
   Modifications:
     - Uses 'command' with PASM to load ClockDelay and ClockState variables into cog.
     - spi_start parameter 'clockdelay_ns' is in nanosecs and function calculates ClockDelay value.
     - device->command is either 0 or packed with 2 or 3 numbers to be used by pasm. 
     - command = 0 -- pasm in idle loop. 
      - In spi_start: 
       1. the number 4 is shifted left 16 bits to indicate 'startup'. 
       2. the ClockDelay value is shifted left 2 bits.
       3. the ClockState value is in bit 0.  All 3 values are or'd. 
      - In spi_in the number 2 is shifted left 16 bits, this is or'd with the address of the Dpin. 
      - In spi_out, the number 1 is shifted left 16 bits, and or'd with the address of the Dpin. 
*/
//     -- In spi_in the number 2 is shifted left 16 bits, this is or'd with the address of the Dpin. 
//    -- In spi_out, the number 1 is shifted left 16 bits, and or'd with the address of the Dpin. 

#ifndef spiasm_H
#define spiasm_H

#if defined(__cplusplus)
extern "C" {
#endif

#include "simpletools.h"                      // Library includes

#define _Title_ 0 ///<     **************** SPI Modes ****************
        // For receiving data from SPI device (spi_in)

#define MPRE 0        ///<  Rx Most Significant Bit first - data valid before clock 
#define LPRE 1        ///<  Rx Least Significant Bit first - data valid before clock 
#define MPOST 2       ///<  Rx Most Significant Bit first - data valid after clock 
#define LPOST 3       ///<  Rx Least Significant Bit first - data valid after clock 

       // For sending data to SPI device (spi_out)

#define LFIRST 4      ///<  Tx Least Significant Bit first ; data valid after clock
#define MFIRST 5      ///<  Tx Most Significant Bit first ; data valid after clock
 
typedef struct spi_struct        // Structure defines variable order in memory
 {                               // each variable address is 1 long from previous
   volatile int command;  ///< packed SHIFTIN/SHIFTOUT command and address of Dpin        
   int Dpin;              ///< Propeller pin for Data-in or Data-out.
   int Cpin;              ///< Propeller pin for Clock. 
   volatile int Mode;     ///< SPI Mode (0 - 5).
   volatile int Bits;     ///< Bits to send or receive.
   volatile int Value;    ///< Data sent or received.
   volatile int Flag;     ///< 0 = Data shift to Prop complete, 1 = in progress
   int cog;               ///< Remembers cog for spi_stop 
 } spia_t;                ///< Data type for struct

// function prototypes
 
    // these prototypes go in library h file (spi.h)
/**
* @brief Starts the spi cog, loads ClockState and ClockDelay PASM variables
* 
* @param clockstate         - 0 = start clock low, 1 = start clock high
*
* @param clockdelay_ns    - Actual delay in nanoseconds (1000ns = 1 usec), not less than 300ns.
*                            - This is different than the spin object that uses ClockDelay.
*                            - spi_start calculates the ClockDelay value from clockdelay_ns.
*                            - ClockDelay = ((clockdelay_ns) - 250ns)) / 50ns.
*                             - e.g. for 2 usec (i.e. 2000 ns): ClockDelay will equal 35.
*                             - for 1 usec ClockDelay will equal 15.
*
* @returns spia_t pointer for use as an identifier for spi functions 
*/
spia_t *spi_start(int clockdelay_ns, int clockstate);

/**
@brief Stop and Release Cog.
@param *device   SPI Device ID returned by spi_start
*/  
void spi_stop(spia_t *device); 

/**
@brief Send data to SPI device
@param *device   SPI Device ID returned by spi_start
@param DQ     Data out pin
@param CLK    Clock pin
@param MD     SPI SHIFTOUT Mode (4 or 5)
@param BIT    Number of bits to send
@param VLU    Data Value to send
*/
void spi_out(spia_t *device, int DQ, int CLK, int MD, int BIT, int VLU);

/**
@brief Receive data from SPI device
@param *device   SPI Device ID returned by spi_start
@param DQ     Data in pin
@param CLK    Clock pin
@param MD     SPI SHIFTIN Mode (0, 1, 2, 3)
@param BIT    Number of bits to receive
@returns      Value sent by SPI device
*/  
int spi_in(spia_t *device,int DQ, int CLK, int MD, int BIT);

#if defined(__cplusplus)
}
#endif
/* __cplusplus */  
#endif

/* spiasm_H */  

/**
 * TERMS OF USE: MIT License
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
